import { e as StepExecutionContext, f as StepAction, R as RetryConfig, g as MastraPrimitives, h as StepResult, W as WorkflowOptions, I as IAction, i as StepVariableType, j as StepConfig, k as WorkflowRunState, l as StepGraph } from './base-CbCmDLUb.cjs';
import { Span } from '@opentelemetry/api';
import { z } from 'zod';
import { M as MastraBase } from './base-hs9NDAZ2.cjs';

declare class Step<TStepId extends string = any, TSchemaIn extends z.ZodSchema | undefined = undefined, TSchemaOut extends z.ZodSchema | undefined = undefined, TContext extends StepExecutionContext<TSchemaIn> = StepExecutionContext<TSchemaIn>> implements StepAction<TStepId, TSchemaIn, TSchemaOut, TContext> {
    id: TStepId;
    description?: string;
    inputSchema?: TSchemaIn;
    outputSchema?: TSchemaOut;
    payload?: TSchemaIn extends z.ZodSchema ? Partial<z.infer<TSchemaIn>> : unknown;
    execute?: (context: TContext) => Promise<TSchemaOut extends z.ZodSchema ? z.infer<TSchemaOut> : unknown>;
    retryConfig?: RetryConfig;
    mastra?: MastraPrimitives;
    constructor({ id, description, execute, payload, outputSchema, inputSchema, retryConfig, }: StepAction<TStepId, TSchemaIn, TSchemaOut, TContext>);
}
declare function createStep<TId extends string, TSchemaIn extends z.ZodSchema | undefined, TSchemaOut extends z.ZodSchema | undefined, TContext extends StepExecutionContext<TSchemaIn>>(opts: StepAction<TId, TSchemaIn, TSchemaOut, TContext>): Step<TId, TSchemaIn, TSchemaOut, TContext>;

interface WorkflowResultReturn<T extends z.ZodType<any>> {
    runId: string;
    start: (props?: {
        triggerData?: z.infer<T>;
    } | undefined) => Promise<{
        triggerData?: z.infer<T>;
        results: Record<string, StepResult<any>>;
        runId: string;
        activePaths: Map<string, {
            status: string;
            suspendPayload?: any;
        }>;
    }>;
}

declare class Workflow<TSteps extends Step<any, any, any>[] = any, TTriggerSchema extends z.ZodType<any> = any> extends MastraBase {
    #private;
    name: string;
    triggerSchema?: TTriggerSchema;
    /**
     * Creates a new Workflow instance
     * @param name - Identifier for the workflow (not necessarily unique)
     * @param logger - Optional logger instance
     */
    constructor({ name, triggerSchema, retryConfig, mastra }: WorkflowOptions<TTriggerSchema>);
    step<TStep extends IAction<any, any, any, any>, CondStep extends StepVariableType<any, any, any, any>, VarStep extends StepVariableType<any, any, any, any>>(step: TStep, config?: StepConfig<TStep, CondStep, VarStep, TTriggerSchema>): this;
    then<TStep extends IAction<any, any, any, any>, CondStep extends StepVariableType<any, any, any, any>, VarStep extends StepVariableType<any, any, any, any>>(step: TStep, config?: StepConfig<TStep, CondStep, VarStep, TTriggerSchema>): this;
    after<TStep extends IAction<any, any, any, any>>(step: TStep): Omit<typeof this, "then" | "after">;
    /**
     * Executes the workflow with the given trigger data
     * @param triggerData - Initial data to start the workflow with
     * @returns Promise resolving to workflow results or rejecting with error
     * @throws Error if trigger schema validation fails
     */
    createRun(): WorkflowResultReturn<TTriggerSchema>;
    /**
     * Rebuilds the machine with the current steps configuration and validates the workflow
     *
     * This is the last step of a workflow builder method chain
     * @throws Error if validation fails
     *
     * @returns this instance for method chaining
     */
    commit(): this;
    getExecutionSpan(runId: string): Span | undefined;
    getState(runId: string): Promise<WorkflowRunState | null>;
    watch(onTransition: (state: WorkflowRunState) => void): () => void;
    resume({ runId, stepId, context: resumeContext, }: {
        runId: string;
        stepId: string;
        context?: Record<string, any>;
    }): Promise<{
        triggerData?: z.TypeOf<TTriggerSchema> | undefined;
        results: Record<string, StepResult<any>>;
        activePaths: Map<string, {
            status: string;
            suspendPayload?: any;
        }>;
    } | undefined>;
    _resume({ runId, stepId, context: resumeContext, }: {
        runId: string;
        stepId: string;
        context?: Record<string, any>;
    }): Promise<{
        triggerData?: z.TypeOf<TTriggerSchema> | undefined;
        results: Record<string, StepResult<any>>;
        activePaths: Map<string, {
            status: string;
            suspendPayload?: any;
        }>;
    } | undefined>;
    __registerPrimitives(p: MastraPrimitives): void;
    get stepGraph(): StepGraph;
    get stepSubscriberGraph(): Record<string, StepGraph>;
    get steps(): Record<string, IAction<any, any, any, any, unknown>>;
}

export { Step as S, Workflow as W, createStep as c };
